﻿/*
 * Copyright (c) 2008-2012, RF-Embedded GmbH
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO.Ports;
using System.Net.Sockets;
using System.IO;
using System.Net;


namespace Test
{
    public class TCPDevice : CSrfeReaderInterface.device.IProtocolDeviceInterface
    {
        Socket tcpClient;
        IPAddress ipAdress;
        Int32 port;
        Thread receiveThread;

        bool terminate;
        

        public TCPDevice(IPAddress host, int port)
        {   
            IPEndPoint endpoint = new IPEndPoint(host, port);
            tcpClient = new Socket(endpoint.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
           
            //Create a thread which executes the ReceiveData-method defined below
            receiveThread = new Thread(new ThreadStart(ReceiveData));
            //Terminate flag for thread
            terminate = false;

            this.ipAdress = host;
            this.port = port;
        }

        //Sends byte data through the given tcp socket
        public override bool Send(byte[] data)
        {
            try
            {
                tcpClient.Send(data);
            }
            catch (Exception /*exception*/)
            {
                return false;
            }
            return true;

        }


        //Method should be called by a thread
        public void ReceiveData()
        {            
            byte[] buf = new byte[256];
            while (!terminate)
            {
                try
                {
                    if (tcpClient.Available > 0)
                    {
                        int numBytes = tcpClient.Receive(buf);

                        //resize the buffer to the real length of received byte[]
                        Array.Resize<byte>(ref buf, numBytes);

                        //report read event
                        RaiseDataReadEvent(buf);
                        Array.Clear(buf, 0, buf.Length);
                    }
                }
                catch (Exception /*exception*/)
                {
                    
                }
            }
        }

        public override bool Open()
        {
            try
            {
                //Connect tcp socket
                tcpClient.Connect(ipAdress, port);
                terminate = false;
                //On successful connection start receiver thread
                if (tcpClient.Connected)
                    receiveThread.Start();
            }
            catch (SocketException /*socketException*/)
            {
                terminate = true;
                return false;
            }

            return tcpClient.Connected;
        }

        //Closes the TCPDevice
        public override bool Close()
        {     
            terminate = true;

            tcpClient.Close();
            return !(tcpClient.Connected);
        }
    }
}
